/////////////////////////////////////////////////////////////////////////////
//
//	Project		:	Active Commentary v1.0 (Visual C++ Dev Studio AddIn)
//	File		:	Commands.cpp
//	Description	:	Implementation of all addin commands 
//					(CCommands class, CProjectData class, CComment class)
//
//	Created and modified by Andrey Levin
//	mail to: Levin@artis-inventum.de
/////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "ActiveComments.h"
#include "Commands.h"
#include <comdef.h>
#include "AboutDlg.h"
#include "CommentaryDlg.h"
#include "ListDlg.h"
#include "CodeTemplatesDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

CString ProjectName, ProjectPath;
CStringArray CmList;
CMapStringToString CodeTemplates;
/////////////////////////////////////////////////////////////////////////////
// Class CProjectData Constructor and Destructor
IMPLEMENT_SERIAL(CProjectData, CObject, 0)

CProjectData::CProjectData()
{
	m_History.RemoveAll();
	m_nVersion = 0;
}

CProjectData::~CProjectData()
{
	DeleteMaps();
	m_History.RemoveAll();
}

BOOL CProjectData::DeleteMaps()
{
	POSITION pos1,pos2;
	CString key1,key2;
	CMapStringToOb *pDoc;
	CComment *pComm;

	try
	{
		for( pos1 = m_CurrentProject.GetStartPosition(); pos1 != NULL; )
		{
			m_CurrentProject.GetNextAssoc( pos1, key1, (CObject*&)pDoc);
			if(pDoc != NULL)
			{
				for( pos2 = pDoc->GetStartPosition(); pos2 != NULL; )
				{
					pDoc->GetNextAssoc( pos2, key2, (CObject*&)pComm);
					delete pComm;
					pComm = NULL;
				}
				pDoc->RemoveAll();
				delete pDoc;
				pDoc = NULL;
			}
		}
		m_CurrentProject.RemoveAll();
	}
	catch(...)
	{
		return FALSE;
	}
	return TRUE;
}

void CProjectData::Serialize(CArchive &ar)
{
	
	CObject::Serialize(ar);

	if(ar.IsStoring())
	{
		m_nVersion = AC_CURRENTVERSION;
		ar << m_nVersion;
	}
	if(ar.IsLoading())
	{
		ar >> m_nVersion;
		if(m_nVersion != AC_CURRENTVERSION)
		{
			TRACE("Previous version.");
			return;
		}
	}
	m_CurrentProject.Serialize(ar);
	m_History.Serialize(ar);
}

/////////////////////////////////////////////////////////////////////////////
// Class CComment Constructor and Destructor

IMPLEMENT_SERIAL(CComment, CObject, 0)

CComment::CComment()
{
	m_szLink = "";
	m_szDocumentName = "";
	m_szProtocol = "";
	m_nTimeOfCreate = 0;
}

CComment::CComment(long Time,CString DocumentName, CString Link, CString Protocol)
{
	m_nTimeOfCreate	= Time;
	m_szLink = Link;
	m_szDocumentName = DocumentName;
	m_szProtocol = Protocol;
}

CComment::~CComment()
{

}

void CComment::Serialize(CArchive &ar)
{
	CObject::Serialize(ar);
	if(ar.IsStoring())
	{
		ar << m_nTimeOfCreate;
		ar << m_szDocumentName << m_szLink << m_szProtocol;
	}

	try
	{
		if(ar.IsLoading())
		{
			ar >> m_nTimeOfCreate;
			ar >> m_szDocumentName >> m_szLink >> m_szProtocol;
		}
	}
	catch(...)
	{
		m_nTimeOfCreate = 0;	
		m_szDocumentName = "";
		m_szLink = "";
		m_szProtocol = "";
	}
}


/////////////////////////////////////////////////////////////////////////////
// CCommands

CCommands::CCommands()
{
	m_pApplication = NULL;
	m_pApplicationEventsObj = NULL;
	m_pDebuggerEventsObj = NULL;
	m_pCurrentDocument = NULL;
	m_bFreeFromAConrtol = FALSE;
	ProjectName = "";

	FillCodeTemplList(&CodeTemplates);
}

CCommands::~CCommands()
{
	TRACE("\n************ Destructor ************\n");

	ProjectName = "";
	ASSERT (m_pApplication != NULL);
	VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));
	m_pApplication->Release();
}	 

void CCommands::SetApplicationObject(IApplication* pApplication)
{
	// This function assumes pApplication has already been AddRef'd
	//  for us, which CDSAddIn did in its QueryInterface call
	//  just before it called us.
	m_pApplication = pApplication;

	// Create Application event handlers
	XApplicationEventsObj::CreateInstance(&m_pApplicationEventsObj);
	m_pApplicationEventsObj->AddRef();
	m_pApplicationEventsObj->Connect(m_pApplication);
	m_pApplicationEventsObj->m_pCommands = this;

	// Create Debugger event handler
	CComPtr<IDispatch> pDebugger;
	if (SUCCEEDED(m_pApplication->get_Debugger(&pDebugger)) 
		&& pDebugger != NULL)
	{
		XDebuggerEventsObj::CreateInstance(&m_pDebuggerEventsObj);
		m_pDebuggerEventsObj->AddRef();
		m_pDebuggerEventsObj->Connect(pDebugger);
		m_pDebuggerEventsObj->m_pCommands = this;
	}
}

void CCommands::UnadviseFromEvents()
{
	ASSERT (m_pApplicationEventsObj != NULL);
	m_pApplicationEventsObj->Disconnect(m_pApplication);
	m_pApplicationEventsObj->Release();
	m_pApplicationEventsObj = NULL;

	if (m_pDebuggerEventsObj != NULL)
	{
		// Since we were able to connect to the Debugger events, we
		//  should be able to access the Debugger object again to
		//  unadvise from its events (thus the VERIFY_OK below--see stdafx.h).
		CComPtr<IDispatch> pDebugger;
		VERIFY_OK(m_pApplication->get_Debugger(&pDebugger));
		ASSERT (pDebugger != NULL);
		m_pDebuggerEventsObj->Disconnect(pDebugger);
		m_pDebuggerEventsObj->Release();
		m_pDebuggerEventsObj = NULL;
	}
}


/////////////////////////////////////////////////////////////////////////////
// Event handlers

// TODO: Fill out the implementation for those events you wish handle
//  Use m_pCommands->GetApplicationObject() to access the Developer
//  Studio Application object

// Application events

HRESULT CCommands::XApplicationEvents::BeforeBuildStart()
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	return S_OK;
}

HRESULT CCommands::XApplicationEvents::BuildFinish(long nNumErrors, long nNumWarnings)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	return S_OK;
}

HRESULT CCommands::XApplicationEvents::BeforeApplicationShutDown()
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	if(m_pCommands != NULL)
		m_pCommands->m_Project.DeleteMaps();
	return S_OK;
}

HRESULT CCommands::XApplicationEvents::DocumentOpen(IDispatch* theDocument)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	CComBSTR bstr;
	CString DocumentName;

	if (!theDocument)
		return S_OK;
	
	((IGenericDocument*)theDocument)->get_Name(&bstr);
	DocumentName = bstr;
	CString ext = GetFileExt(DocumentName);
	if((ext == "rc")||(ext == "rc2"))
		return S_OK;
	if(!m_pCommands->m_Project.m_CurrentProject.Lookup(DocumentName, (CObject*&)m_pCommands->m_pCurrentDocument))
	{
		m_pCommands->m_pCurrentDocument = new CMapStringToOb;
		m_pCommands->m_Project.m_CurrentProject.SetAt(DocumentName,(CObject*&)m_pCommands->m_pCurrentDocument);
	}
	
	return S_OK;
}

HRESULT CCommands::XApplicationEvents::BeforeDocumentClose(IDispatch* theDocument)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	return S_OK;
}

HRESULT CCommands::XApplicationEvents::DocumentSave(IDispatch* theDocument)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	TRACE("\n************ DocumentSave ************\n");
	if(m_pCommands->m_bFreeFromAConrtol)
		m_pCommands->m_bFreeFromAConrtol = FALSE;
	else
		if(!ProjectName.IsEmpty())
		{
			CStdioFile File(ProjectName, CFile::modeCreate | CFile::modeWrite | CFile::typeBinary );
			CArchive ar(&File, CArchive::store);
			m_pCommands->m_Project.Serialize(ar);
		}
	
	return S_OK;
}

HRESULT CCommands::XApplicationEvents::NewDocument(IDispatch* theDocument)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	return S_OK;
}

HRESULT CCommands::XApplicationEvents::WindowActivate(IDispatch* theWindow)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	return S_OK;
}

HRESULT CCommands::XApplicationEvents::WindowDeactivate(IDispatch* theWindow)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	return S_OK;
}

HRESULT CCommands::XApplicationEvents::WorkspaceOpen()
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	
	IGenericProject * pProject=NULL;
	IApplication* pApp=NULL;
	CComBSTR bstr;
	CString Path;

	pApp = m_pCommands->GetApplicationObject();

	ASSERT(pApp);

	TRACE("\n************ Open ************\n");
	pApp->get_ActiveProject((IDispatch**)&pProject);

	if (!pProject)
		return S_OK;
	
	pProject->get_Name(&bstr);
	ProjectName = bstr;
	pProject->get_FullName(&bstr);
	Path = GetFilePath((CString)bstr) + AC_PATH;
	ProjectName = Path + ProjectName + ".acp";
	if(GetFileAttributes(ProjectName) != -1)
	{
		CStdioFile File;
		if(File.Open(ProjectName, CFile::modeRead | CFile::typeBinary ))
		{
			CArchive ar(&File, CArchive::load);
			m_pCommands->m_Project.Serialize(ar);
		}
	}
	else
	{
		CreateDirectory(Path,NULL);
	}

	pProject->Release();

	return S_OK;
}

HRESULT CCommands::XApplicationEvents::WorkspaceClose()
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	TRACE("\n************ Close ************\n");
	if(m_pCommands->m_bFreeFromAConrtol)
		m_pCommands->m_bFreeFromAConrtol = FALSE;
	else
		if((!ProjectName.IsEmpty())&&(!m_pCommands->m_Project.m_CurrentProject.IsEmpty()))
		{
			CStdioFile File(ProjectName, CFile::modeCreate | CFile::modeWrite | CFile::typeBinary );
			CArchive ar(&File, CArchive::store);
			m_pCommands->m_Project.Serialize(ar);
			ProjectName = "";
			ProjectPath = "";
			m_pCommands->m_Project.DeleteMaps();
			m_pCommands->m_Project.m_History.RemoveAll();

		}

	return S_OK;
}

HRESULT CCommands::XApplicationEvents::NewWorkspace()
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	return S_OK;
}

// Debugger event

HRESULT CCommands::XDebuggerEvents::BreakpointHit(IDispatch* pBreakpoint)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	return S_OK;
}


/////////////////////////////////////////////////////////////////////////////
// CCommands methods

////////////////////////////////////////////////////////////
// About ActiveComment AddIn
STDMETHODIMP CCommands::About()
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	VERIFY_OK(m_pApplication->EnableModeless(VARIANT_FALSE));
	CAboutDlg dlg;
	dlg.DoModal();
	VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));

	return S_OK;  
}

////////////////////////////////////////////////////////////
//  Active Comment
STDMETHODIMP CCommands::CommentSetup()
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	VERIFY_OK(m_pApplication->EnableModeless(VARIANT_FALSE));

	CString Title="", Link="", Protocol = "",sAddPrComm = "";
	CString DocumentName="", Selection="", oldTitle="";
	CComBSTR bstr;
	CCommentaryDlg dlg;
	long row,col;

// get pointer to current document
	IGenericDocument* pDoc;
	if(!GetCurrentDocument(m_pApplication,pDoc))
	{
		VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));
		return S_OK;
	}
	
// get filename of the current file
	pDoc->get_Name(&bstr);
	DocumentName = bstr;
	if(!CheckMap())
	{
		pDoc->Release();
		VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));
		return S_OK;
	}

// get pointer to current selection
	ITextSelection* pSel;
	if(!GetCurrentSelection(pDoc,pSel))
	{
		pDoc->Release();
		VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));
		return S_OK;
	}

// if multiple selection -> exit. Current ver. not support this yet.
	pSel->get_BottomLine(&row);
	pSel->get_TopLine(&col);
	if(row!=col)
	{
		pSel->Release();
		pDoc->Release();
		VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));
		return S_OK;
	}

// get cursor current position
	pSel->get_CurrentLine(&row);
	pSel->get_CurrentColumn(&col);
	pSel->get_Text(&bstr);
	Selection = bstr;
	StripString(Selection,"\n");
	StripString(Selection,"	");

// check: contains current string ActiveCommentary?
	Title = GetTitleFromSelection(pSel);
	oldTitle = Title;

	dlg.m_szTitle = (Title.IsEmpty())?Selection:Title;
	dlg.m_pSel = pSel;
	dlg.m_szDocumentName = DocumentName;
	dlg.m_pMap = m_pCurrentDocument;
	dlg.m_pStrArray = &m_Project.m_History;

	m_pComment = NULL;
	if(m_pCurrentDocument->Lookup(Title,(CObject*&) m_pComment))
	{
		dlg.m_szLink = m_pComment->m_szLink;
		dlg.m_szProtocol = m_pComment->m_szProtocol;
		dlg.m_nTime = m_pComment->m_nTimeOfCreate;
	}
	else
	{
		dlg.m_szLink = dlg.m_szTitle;
		if(!oldTitle.IsEmpty())
			if(AfxMessageBox("Can't find this comment in the base. Create? ",MB_YESNO) == IDNO)
			{
				pSel->Release();
				pDoc->Release();
				VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));
				return S_OK;
			}
	}

// change ActiveCommentary properties
	dlg.m_szInitPath = GetFilePath(ProjectName) + "\\";
	if(dlg.DoModal() != IDOK)
	{
		pSel->Release();
		pDoc->Release();
		VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));
		return S_OK;
	}
	SetSelection(row,col,Selection,pSel);
	Title = dlg.m_szTitle;
	Link = dlg.m_szLink;
	Protocol = dlg.m_szProtocol;
	StripString(Title);
	StripString(Link);
	StripString(Protocol);
	if(!m_pCurrentDocument->Lookup(oldTitle,(CObject*&) m_pComment))
	{
		time_t osBinaryTime;  // C run-time time (defined in <time.h>)
		time( &osBinaryTime ) ;  
		m_pComment = new CComment(osBinaryTime,DocumentName,Link,Protocol);
	}
	else
	{
		m_pComment->m_szLink = Link;
		m_pCurrentDocument->RemoveKey(oldTitle);
	}
	m_pCurrentDocument->SetAt(Title,m_pComment);
	
	if(oldTitle.Compare(Title) == 0)
	{
		pDoc->Release();
		pSel->Release();
		VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));
		return S_OK;
	}

	if(IsSelectionInTheCommentBlock(pSel) == SEARCH_ISNO)
		sAddPrComm = "// ";
	
// Remove old title (if exists)	
	if(!oldTitle.IsEmpty())
	{
		_variant_t DelType,SelType;
		VARIANT_BOOL ret;
	
		DelType.iVal = dsFirstColumn;
		DelType.vt = VT_I2;
		SelType.iVal = dsMove;
		SelType.vt = VT_I2;
		pSel->StartOfLine(DelType,SelType);
		SelType.iVal = NULL;
		oldTitle = 	"[[->" + oldTitle + "]]";
		bstr = oldTitle.AllocSysString();
		pSel->FindText(bstr,SelType,&ret);
		oldTitle.ReleaseBuffer();
		DelType.iVal = 1;
		DelType.vt = VT_I2;
		if(ret)
			pSel->Delete(DelType);
	}

	Title = sAddPrComm + "[[->" + Title + "]]";
	bstr = Title.AllocSysString();
	pSel->put_Text(bstr);
	Title.ReleaseBuffer();	

	pDoc->Release();
	pSel->Release();
	VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));
	return S_OK;
}

////////////////////////////////////////////////////////////
// View ActiveComment
STDMETHODIMP CCommands::CommentView()
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())
	VERIFY_OK(m_pApplication->EnableModeless(VARIANT_FALSE));

// get pointer to current document
	IGenericDocument* pDoc;
	if (!GetCurrentDocument(m_pApplication,pDoc))
	{
		VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));
		return S_OK;
	}
	
	if(!CheckMap())
	{
		pDoc->Release();
		VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));
		return S_OK;
	}

// get pointer to current selection
	ITextSelection* pSel;
	if (!GetCurrentSelection(pDoc,pSel))
	{
		pDoc->Release();
		VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));
		return S_OK;
	}

// if multiple selection -> exit. Current ver. not support this yet.
	long row1,row2;
	pSel->get_BottomLine(&row1);
	pSel->get_TopLine(&row2);
	if(row1!=row2)
	{
		pDoc->Release();
		pSel->Release();
		VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));
		return S_OK;
	}

// check: contents current string ActiveCommentary?
	CString Title = GetTitleFromSelection(pSel);
	m_pComment = NULL;
	
	if((Title.IsEmpty()) ||(!m_pCurrentDocument->Lookup(Title,(CObject*&) m_pComment)))
	{
		pSel->Release();
		pDoc->Release();
		CommentSetup();
		VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));		
		return S_OK;
	}	

	ExecuteLink(m_pComment);

	pDoc->Release();
	pSel->Release();
	VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));
	return S_OK;
}

////////////////////////////////////////////////////////////
// View in ListDlg list of all Active Comments in Document
STDMETHODIMP CCommands::ViewAllComments()
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	VERIFY_OK(m_pApplication->EnableModeless(VARIANT_FALSE));

	CListDlg dlg;

// get pointer to current document
	IGenericDocument* pDoc;
	if (!GetCurrentDocument(m_pApplication,pDoc))
	{
		VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));
		return S_OK;
	}
	
	if(!CheckMap())
	{
		pDoc->Release();
		VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));
		return S_OK;
	}

// get pointer to current selection
	ITextSelection* pSel;
	if (!GetCurrentSelection(pDoc,pSel))
	{
		pDoc->Release();
		VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));
		return S_OK;
	}

	dlg.m_pProjectMap = &m_Project.m_CurrentProject;
	dlg.m_pApp = m_pApplication;
	dlg.m_pStrArray = &m_Project.m_History;

	pDoc->Release();
	pSel->Release();
	if(dlg.DoModal() != IDOK)
	{
		VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));
		return S_OK;
	}

	VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));

	return S_OK;
}

////////////////////////////////////////////////////////////
// Clear Active Comment 
STDMETHODIMP CCommands::CommentClear()
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	VERIFY_OK(m_pApplication->EnableModeless(VARIANT_FALSE));

	CString Title="",Selection = "";
	CComBSTR bstr;
	_variant_t DelType,SelType;
	VARIANT_BOOL ret;

// get pointer to current document
	IGenericDocument* pDoc;
	if (!GetCurrentDocument(m_pApplication,pDoc))
	{
		VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));
		return S_OK;
	}

	if(!CheckMap())
	{
		pDoc->Release();
		VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));
		return S_OK;
	}

// get pointer to current selection
	ITextSelection* pSel;
	if (!GetCurrentSelection(pDoc,pSel))
	{
		pDoc->Release();
		VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));
		return S_OK;
	}

// if multiple selection -> exit. Current ver. not support this yet.
	long row1,row2;
	pSel->get_BottomLine(&row1);
	pSel->get_TopLine(&row2);
	if(row1!=row2)
	{
		pSel->Release();
		pDoc->Release();
		VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));
		return S_OK;
	}
	
// check: contents current string ActiveCommentary?
	Title = GetTitleFromSelection(pSel);

	m_pComment = NULL;
	if((Title.IsEmpty()) &&(!m_pCurrentDocument->Lookup(Title,(CObject*&) m_pComment)))
	{
		pSel->Release();
		pDoc->Release();
		VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));
		return S_OK;
	}
	
	m_pCurrentDocument->RemoveKey(Title);
	delete m_pComment;	

	DelType.iVal = dsFirstColumn;
	DelType.vt = VT_I2;
	SelType.iVal = dsMove;
	SelType.vt = VT_I2;
	Selection = "[[->" + Title + "]]";
	pSel->StartOfLine(DelType,SelType);
	SelType.iVal = NULL;
	bstr = Selection.AllocSysString();
	pSel->FindText(bstr,SelType,&ret);
	Selection.ReleaseBuffer();
	if(!ret)
	{
		pSel->Release();
		pDoc->Release();
		VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));
		return S_OK;
	}

	DelType.iVal = 1;
	DelType.vt = VT_I2;
	pSel->Delete(DelType);
	bstr = Title.AllocSysString();
	pSel->put_Text(bstr);
	Title.ReleaseBuffer();

	pDoc->Release();
	pSel->Release();
	VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));

	return S_OK;
}


////////////////////////////////////////////////////////////
// Delete Active Comment
STDMETHODIMP CCommands::CommentDelete()
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	VERIFY_OK(m_pApplication->EnableModeless(VARIANT_FALSE));

	CString Title="";
	CComBSTR bstr;
	_variant_t DelType,SelType;
	VARIANT_BOOL ret;

// get pointer to current document
	IGenericDocument* pDoc;
	if (!GetCurrentDocument(m_pApplication,pDoc))
	{
		VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));
		return S_OK;
	}

	if(!CheckMap())
	{
		pDoc->Release();
		VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));
		return S_OK;
	}

// get pointer to current selection
	ITextSelection* pSel;
	if (!GetCurrentSelection(pDoc,pSel))
	{
		pDoc->Release();
		VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));
		return S_OK;
	}
	
// if multiple selection -> exit. Current ver. not support this yet.
	long row1,row2;
	pSel->get_BottomLine(&row1);
	pSel->get_TopLine(&row2);
	if(row1!=row2)
	{
		pSel->Release();
		pDoc->Release();
		VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));
		return S_OK;
	}

// check: contents current string ActiveCommentary?
	Title = GetTitleFromSelection(pSel);
	m_pComment = NULL;

	if((Title.IsEmpty()) &&(!m_pCurrentDocument->Lookup(Title,(CObject*&) m_pComment)))
	{
		pSel->Release();
		pDoc->Release();
		VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));
		return S_OK;
	}
	
	m_pCurrentDocument->RemoveKey(Title);
	delete m_pComment;	

	DelType.iVal = dsFirstColumn;
	DelType.vt = VT_I2;
	SelType.iVal = dsMove;
	SelType.vt = VT_I2;
	Title = "[[->" + Title + "]]";
	pSel->StartOfLine(DelType,SelType);
	SelType.iVal = NULL;
	bstr = Title.AllocSysString();
	pSel->FindText(bstr,SelType,&ret);
	Title.ReleaseBuffer();
	if(!ret)
	{
		pSel->Release();
		pDoc->Release();
		VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));
		return S_OK;
	}

	DelType.iVal = 1;
	DelType.vt = VT_I2;
	pSel->Delete(DelType);

	pDoc->Release();
	pSel->Release();
	VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));

	return S_OK;
}

////////////////////////////////////////////////////////////
// Change case of selection to upper
STDMETHODIMP CCommands::UpperCase()
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	VERIFY_OK(m_pApplication->EnableModeless(VARIANT_FALSE));
	IGenericDocument* pDoc;
	if (!GetCurrentDocument(m_pApplication,pDoc))
	{
		VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));
		return S_OK;
	}

	if(!CheckMap())
	{
		pDoc->Release();
		VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));
		return S_OK;
	}

	ITextSelection* pSel;
	if (!GetCurrentSelection(pDoc,pSel))
	{
		pDoc->Release();
		VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));
		return S_OK;
	}

	pSel->ChangeCase(dsUppercase);

	pSel->Release();
	pDoc->Release();

	VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));

	return S_OK;
}

////////////////////////////////////////////////////////////
// Change case of selection to lower
STDMETHODIMP CCommands::LowerCase()
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	VERIFY_OK(m_pApplication->EnableModeless(VARIANT_FALSE));
	IGenericDocument* pDoc;
	if (!GetCurrentDocument(m_pApplication,pDoc))
	{
		VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));
		return S_OK;
	}

	if(!CheckMap())
	{
		pDoc->Release();
		VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));
		return S_OK;
	}

	ITextSelection* pSel;
	if (!GetCurrentSelection(pDoc,pSel))
	{
		pDoc->Release();
		VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));
		return S_OK;
	}

	pSel->ChangeCase(dsLowercase);
		
	pDoc->Release();
	pSel->Release();
	VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));

	return S_OK;
}

////////////////////////////////////////////////////////////
// Capitalize selection
STDMETHODIMP CCommands::Capitalize()
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	VERIFY_OK(m_pApplication->EnableModeless(VARIANT_FALSE));
	IGenericDocument* pDoc;
	if (!GetCurrentDocument(m_pApplication,pDoc))
	{
		VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));
		return S_OK;
	}

	if(!CheckMap())
	{
		pDoc->Release();
		VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));
		return S_OK;
	}

	ITextSelection* pSel;
	if (!GetCurrentSelection(pDoc,pSel))
	{
		pDoc->Release();
		VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));
		return S_OK;
	}

	pSel->ChangeCase(dsCapitalize);

	pSel->Release();
	pDoc->Release();

	VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));

	return S_OK;
}

////////////////////////////////////////////////////////////
// Converts spaces to tabs in a selection.
STDMETHODIMP CCommands::ConvertSpaceToTab()
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

	VERIFY_OK(m_pApplication->EnableModeless(VARIANT_FALSE));

	IGenericDocument* pDoc;
	if (!GetCurrentDocument(m_pApplication,pDoc))
	{
		VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));
		return S_OK;
	}

	if(!CheckMap())
	{
		pDoc->Release();
		VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));
		return S_OK;
	}

	ITextSelection* pSel;
	if (!GetCurrentSelection(pDoc,pSel))
	{
		pDoc->Release();
		VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));
		return S_OK;
	}

	long row, col;
	CComBSTR bstr;
	pSel->get_CurrentLine(&row);
	pSel->get_CurrentColumn(&col);
	pSel->get_Text(&bstr);
	if(((CString)bstr).IsEmpty())
		pSel->SelectAll();
	pSel->Tabify();

	pSel->Release();
	pDoc->Release();
	VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));

	return S_OK;
}

////////////////////////////////////////////////////////////
// Check: exists or not exists CMapStringToOb on the current document
BOOL CCommands::CheckMap()
{
	IGenericDocument* pDoc;
	CComBSTR bstr;
	CString DocumentName;

	TRACE("\n************ CheckMap ************\n");
	if(ProjectName.IsEmpty())
		m_pApplicationEventsObj->WorkspaceOpen();
	
	m_pApplication->get_ActiveDocument((IDispatch**)&pDoc);
	if (!pDoc)
		return FALSE;
	
	pDoc->get_Name(&bstr);
	DocumentName = bstr;

	CString ext = GetFileExt(DocumentName);
	ext.MakeLower();
	if((ext != "c")&&(ext != "cpp")&&
		(ext != "h")&&(ext != "hpp")&&
		(ext != "cxx")&&(ext != "hxx")&&
		(ext != "tli")&&(ext != "tlh")&&
		(ext != "inl")&&(ext != "txt")&&
		(ext != "odl")&&(ext != "idl"))
	{
		pDoc->Release();
		return FALSE;
	}

	if(!m_Project.m_CurrentProject.Lookup(DocumentName, (CObject*&)m_pCurrentDocument))
	{
		m_pCurrentDocument = new CMapStringToOb;
		m_Project.m_CurrentProject.SetAt(DocumentName,(CObject*&)m_pCurrentDocument);
	}
	pDoc->Release();
	return TRUE;
}

////////////////////////////////////////////////////////////
// Add code template to the current selection
STDMETHODIMP CCommands::AddCodeTemplate()
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())
	VERIFY_OK(m_pApplication->EnableModeless(VARIANT_FALSE));

	IGenericDocument* pDoc;
	if (!GetCurrentDocument(m_pApplication,pDoc))
	{
		VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));
		return S_OK;
	}
	
	if(!CheckMap())
	{
		pDoc->Release();
		VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));
		return S_OK;
	}

	ITextSelection* pSel;
	if (!GetCurrentSelection(pDoc,pSel))
	{
		pDoc->Release();
		VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));
		return S_OK;
	}

	CComBSTR bstr;
	CString sTemp="",StrToInsert="";

	pSel->get_Text(&bstr);

	CCodeTemplatesDlg dlg;
	if(dlg.DoModal() != IDOK)
	{
		pSel->Release();
		pDoc->Release();
		VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));
		return S_OK;
	}
	CodeTemplates.Lookup(dlg.m_szTemplate,sTemp);
	if(dlg.m_szTemplate.Compare("class declaration"))
		StrToInsert.Format(sTemp,(CString)bstr);
	else
		StrToInsert.Format(sTemp,(CString)bstr,(CString)bstr,(CString)bstr);
	bstr = StrToInsert.AllocSysString();
	pSel->put_Text(bstr);
	StrToInsert.ReleaseBuffer();

	pSel->Release();
	pDoc->Release();
	VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));
	return S_OK;
}

/////////////////////////////////////////////////////////////////////////////
// Common Functions

// Check current selection : has it ActiveComment or no?
// Return : Title of ActiveComment (if YES) or empty string (if NO)
CString GetTitleFromSelection(ITextSelection* pSel)
{
	CString strResult=AC_WORKINGSTRING, s,Selected;
	CComBSTR bstr;
	_variant_t SelType, SelCount;
	long row,col;
	BOOL prExit = FALSE;
	
	pSel->get_Text(&bstr);	
	Selected = bstr;
	pSel->get_CurrentColumn(&col);
	pSel->get_CurrentLine(&row);

	SelType.iVal = dsExtend;
	SelCount.lVal = 1;
	SelCount.vt = VT_I4;
	SelType.vt = VT_I2;
	
	do
	{
		pSel->get_Text(&bstr);
	 	s = bstr;
		StripString(s,"\n");
		StripString(s,"	");
		if(strResult.Compare(s))
			strResult = s;
		else
			prExit = TRUE;
		if(strResult.Find("[[->")==-1)
			pSel->CharLeft(SelType,SelCount);
		else
			prExit = TRUE;
	}
	while (!prExit);

	if(strResult.Find("[[->")!=-1)
	{
		s = strResult;
		strResult = AC_WORKINGSTRING;
		do
		{
			pSel->get_Text(&bstr);
			if(strResult.Compare((CString)bstr))
				strResult = bstr;
			else
				prExit = FALSE;
			if(strResult.Find("]]")==-1)
				pSel->CharRight(SelType,SelCount);
			else
				prExit = FALSE;
		}
		while (prExit);
	}
	if((s.Find(strResult) != -1)||
		((s.Find("[[->") != -1)&&(s.Find("]]") != -1)))
		strResult = s;
	else
		if(strResult.Find(s) == -1)
			strResult = s + strResult;
	
	SetSelection(row,col,Selected, pSel);

	strResult = GetTitleFromString(strResult);
	return strResult;
}

////////////////////////////////////////////////////////////
// Check current string: has it ActiveComment
// Return : Title of ActiveComment (if YES) or empty string (if NO)
CString GetTitleFromString(CString FromStr)
{
	CString strResult;
	int posStart = FromStr.Find("[[->");
	int posEnd = FromStr.Find("]]");

	if ((posStart != -1)&&(posEnd != -1))
	{
		strResult = FromStr.Right(FromStr.GetLength() - posStart - 4);
		posEnd = strResult.Find("]]");
		strResult.Delete(posEnd,strResult.GetLength() - posEnd);
	}
	else 
		return "";
	StripString(strResult);
	StripString(strResult,"\n");
	return strResult;
}

////////////////////////////////////////////////////////////
//	Remove spaces at the begin and end of string
void StripString(CString & s, const char *symbols /* =" " */ )
{
    int length = s.GetLength();
    if (!length)
        return;
    int firstpos = strspn(s, symbols);
    if (firstpos == length)
    {
        s.Empty();
        return;
    }
    int lastpos = length - 1;
    while (strchr(symbols, s[lastpos]))
        lastpos--;
    s = s.Mid(firstpos, lastpos - firstpos + 1);
}

////////////////////////////////////////////////////////////
//	restore selection
BOOL SetSelection(long row, long col, CString selection,ITextSelection* pSel)
{
	CComBSTR bstr;
	CString s;
	_variant_t SelType, SelCount;
	long count;

	count = selection.GetLength();
	SelType.iVal = dsMove;
	SelCount.lVal = count;
	SelCount.vt = VT_I4;
	SelType.vt = VT_I2;
	pSel->MoveTo(row,col,SelType);

	if(selection.IsEmpty())
		return TRUE;
	SelType.iVal = dsExtend;
	pSel->CharRight(SelType,SelCount);
	pSel->get_Text(&bstr);
	s=bstr;
	if(s.Compare(selection))
	{
		SelType.iVal = dsMove;
		pSel->MoveTo(row,col,SelType);
		SelType.iVal = dsExtend;
		pSel->CharLeft(SelType,SelCount);
		pSel->get_Text(&bstr);
		s=bstr;
		if(s.Compare(selection))
			return FALSE;
		else
			return TRUE;
	}
	else
		return TRUE;
}

////////////////////////////////////////////////////////////
//	select commentary
BOOL SelectComment(CString Title, ITextSelection *pSel)
{
	_variant_t SelType;
	VARIANT_BOOL ret;
	CComBSTR bstr;
	CString s = "[[->" + Title + "]]";

	SelType.iVal = dsMove;
	SelType.vt = VT_I2;
	pSel->StartOfDocument(SelType);
	SelType.iVal = dsMatchCase|dsMatchForward|dsMatchWord;
	bstr = s.AllocSysString();
	pSel->FindText(bstr,SelType,&ret);
	s.ReleaseBuffer();
	return ret;
}
////////////////////////////////////////////////////////////
//	extract file path
CString	GetFilePath(LPCSTR FileName) 
{
	CString strResult = FileName;
	int pos = strResult.ReverseFind('\\');
	
	if (pos >= 0)
		return strResult.Left(pos);

	strResult.Empty();
	return strResult;
}

////////////////////////////////////////////////////////////
//	extract file extentions
CString GetFileExt(LPCSTR FileName) 
{
	CString strResult = FileName;
	int pos = strResult.ReverseFind('.');
	if (pos >= 0)
		return strResult.Right(strResult.GetLength() - pos - 1);
	strResult.Empty();
	return strResult;
}

////////////////////////////////////////////////////////////
//	extract only name of file
void ExtractName(CString &FileName, int mode) 
{
	int pos;
	if((mode == EXTR_WITHOUT_EXT)||(mode == EXTR_WITHOUT_PATHANDEXT))
	{
		pos = FileName.ReverseFind('.');
		if (pos >= 0)
			FileName.Delete(pos, FileName.GetLength() - pos);
	}
	if((mode == EXTR_WITHOUT_PATH)||(mode == EXTR_WITHOUT_PATHANDEXT))
	{
		pos = FileName.ReverseFind('\\');
		if (pos >= 0)
			FileName.Delete(pos, FileName.GetLength() - pos);
	}
}

////////////////////////////////////////////////////////////
//	execute command consisted in recieved pointer CComment*
void ExecuteLink(CComment *pComment)
{
	CString URL, Parameters="";
	if(!pComment->m_szProtocol.CompareNoCase("mk:@MSITStore:"))
	{
		URL = "hh.exe ";
		Parameters = pComment->m_szLink;
	}
	else
		URL = pComment->m_szProtocol + pComment->m_szLink;
	
	ShellExecute(NULL,"open",URL,Parameters,NULL,SW_SHOWNORMAL);
}

////////////////////////////////////////////////////////////
//	Return full ProjectName
CString GetProjectName()
{
	return ProjectName;
}

////////////////////////////////////////////////////////////
//	Open Document
BOOL OpenDocument(IApplication *pApp, CString DocName)
{
	if(DocName.IsEmpty())
		return FALSE;

	IDocuments *pDocs;
	pApp->get_Documents((IDispatch**)&pDocs);
	if (!pDocs)
		return FALSE;
	
	COleVariant type("Auto");
	COleVariant ro((long)FALSE);
	CComBSTR bstr = DocName.AllocSysString();

	IGenericDocument *pDoc=NULL;
	pDocs->Open(bstr, type, ro, (IDispatch**)&pDoc);

	if(!pDoc)
	{
		pDocs->Release();
		return FALSE;
	}

	pDoc->Release();
	pDocs->Release();
	return TRUE;
}

////////////////////////////////////////////////////////////
//	Get active document
BOOL GetCurrentDocument(IApplication *pApp,IGenericDocument*&pDoc)
{
	if(!pApp)
		return FALSE;

	pApp->get_ActiveDocument((IDispatch**)&pDoc);
	if(!pDoc)
		return FALSE;
	else
		return TRUE;
}

////////////////////////////////////////////////////////////
//	Get selection in the active document
BOOL GetCurrentSelection(IGenericDocument* pDoc, ITextSelection*&pSel)
{
	if(!pDoc)
		return FALSE;

	((ITextDocument*)pDoc)->get_Selection((IDispatch**)&pSel);
	if (!pSel)
		return FALSE;
	else
		return TRUE;
}

////////////////////////////////////////////////////////////
//	Find first Comment in the active document
//	Return: row (if found) or -1 (if not found)
long FindCommentaryByTitle(ITextSelection* pSel, CString Title)
{
	if(!pSel)
		return -1;

	long row;
	CString sTemp;
	_variant_t SelType;
	VARIANT_BOOL ret;
	CComBSTR bstr;

	SelType.iVal = dsMove;
	SelType.vt = VT_I2;
	sTemp = "[[->" + Title + "]]";
	pSel->StartOfDocument(SelType);
	SelType.iVal = NULL;
	bstr = sTemp.AllocSysString();
	pSel->FindText(bstr,SelType,&ret);
	sTemp.ReleaseBuffer();
	if(ret)
	{
		pSel->get_CurrentLine(&row);
		return row;
	}
	else
		return -1;
}

BOOL AddToHistory(CStringArray	*pArr, CString Str)
{
	for (int i=0; i < pArr->GetSize(); i++)
		if(!(pArr->GetAt(i)).CompareNoCase(Str))
			return FALSE;
	pArr->InsertAt(0,Str);
	return TRUE;
} 

void FillCodeTemplList(CMapStringToString *pMap)
{
	pMap->SetAt("class declaration","class C%s : public C \n{\nprivate:\n\nprotected:\n\npublic:\n	C%s();\n	~C%s();\n};");
	pMap->SetAt("do..while statement","do\n{\n\n}\nwhile (%s)");
	pMap->SetAt("for statement","for(%s; ; )\n{\n\n}");
	pMap->SetAt("function declaration","void %s( )\n{\n\n}");
	pMap->SetAt("if statement","if(%s)\n{\n\n}");
	pMap->SetAt("if else","if(%s)\n{\n\n}\nelse\n{\n\n}");
	pMap->SetAt("structure declaration","struct %s\n{\n\n};");
	pMap->SetAt("switch statement","switch (%s)\n{\n	case : ;\n		break;\n	case : ;\n		break;\n	default: ;\n}");
	pMap->SetAt("template class declaration","template <class C>class C%s\n{\n};");	
	pMap->SetAt("try catch","try\n{\n%s\n}\ncatch()\n{\n}");
	pMap->SetAt("while statement","while (%s)\n{\n\n}");
}

////////////////////////////////////////////////////////////
// Check selection: this selection located in the comment 
// block or no?
DWORD IsSelectionInTheCommentBlock(ITextSelection *pSel)
{
	CComBSTR bstr, bstrCurSel;
	CString sTemp;
	long oldRow,oldCol,i;
	_variant_t SelType1,SelType2;
	DWORD ret;	

	TRACE("\n************ IsSelectionInTheCommentBlock ************\n");
	if(!pSel)
		return SEARCH_SELECTIONISEMPTY;
	
	pSel->get_Text(&bstrCurSel);
	pSel->get_CurrentLine(&oldRow);
	pSel->get_CurrentColumn(&oldCol);

	SelType1.iVal = dsFirstColumn;
	SelType1.vt = VT_I2;
	SelType2.iVal = dsMove;
	SelType2.vt = VT_I2;
	pSel->StartOfLine(SelType1,SelType2);
	SelType2.iVal = dsExtend;
	pSel->EndOfLine(SelType2);
	pSel->get_Text(&bstr);
	i=((CString)bstr).Find("//");
	if((i < oldCol)&&(i != -1))
		ret	= SEARCH_ISOK;
	else
		ret	= SEARCH_ISNO;

	SetSelection(oldRow,oldCol,(CString)bstrCurSel,pSel);

	return ret;
}
