/////////////////////////////////////////////////////////////////////////////
//
//	Project		:	Active Commentary v1.0 (Visual C++ Dev Studio AddIn)
//	File		:	DSAddIn.cpp
//	Description	:	Implementation of functions which called on loads and 
//					unloads addin (CDSAddIn class)
//
//	Created and modified by Andrey Levin
//
/////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "ActiveComments.h"
#include "DSAddIn.h"
#include "Commands.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// This is called when the user first loads the add-in, and on start-up
//  of each subsequent Developer Studio session
STDMETHODIMP CDSAddIn::OnConnection(IApplication* pApp, VARIANT_BOOL bFirstTime,
		long dwCookie, VARIANT_BOOL* OnConnection)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	// Store info passed to us
	IApplication* pApplication = NULL;
	if (FAILED(pApp->QueryInterface(IID_IApplication, (void**) &pApplication))
		|| pApplication == NULL)
	{
		*OnConnection = VARIANT_FALSE;
		return S_OK;
	}

	m_dwCookie = dwCookie;

	// Create command dispatch, send info back to DevStudio
	CCommandsObj::CreateInstance(&m_pCommands);
	m_pCommands->AddRef();

	// The QueryInterface above AddRef'd the Application object.  It will
	//  be Release'd in CCommand's destructor.
	m_pCommands->SetApplicationObject(pApplication);

	// (see stdafx.h for the definition of VERIFY_OK)

	VERIFY_OK(pApplication->SetAddInInfo((long) AfxGetInstanceHandle(),
		(LPDISPATCH) m_pCommands, IDR_TOOLBAR_MEDIUM, IDR_TOOLBAR_LARGE, m_dwCookie));

	// Inform DevStudio of the commands we implement

	// TODO: Replace the AddCommand call below with a series of calls,
	//  one for each command your add-in will add.

	// The command name should not be localized to other languages.  The
	//  tooltip, command description, and other strings related to this
	//  command are stored in the string table (IDS_CMD_STRING) and should
	//  be localized.

	LPCTSTR szCommand;
	VARIANT_BOOL bRet;
	CString strCmdString;
	{
		szCommand = _T("CommentSetup");
		strCmdString.LoadString(IDS_CMD_COMMENTSETUP);
		strCmdString = szCommand + strCmdString;
		CComBSTR bszCmdString(strCmdString);
		CComBSTR bszMethod(_T("CommentSetup"));
		CComBSTR bszCmdName(szCommand);
		VERIFY_OK(pApplication->AddCommand(bszCmdString, bszMethod, 0, m_dwCookie, &bRet));
		if (bFirstTime == VARIANT_TRUE)
		{
			VERIFY_OK(pApplication->
				AddCommandBarButton(dsGlyph, bszCmdName, m_dwCookie));
			CComBSTR bszKeystroke("ALT+S");
			CComBSTR bszEditor("Main");
			VERIFY(SUCCEEDED(pApplication->
				AddKeyBinding(bszKeystroke, bszCmdName, bszEditor)));

		}
	}
	{
		szCommand = _T("CommentView");
		strCmdString.LoadString(IDS_CMD_COMMENTVIEW);
		strCmdString = szCommand + strCmdString;
		CComBSTR bszCmdString(strCmdString);
		CComBSTR bszMethod(_T("CommentView"));
		CComBSTR bszCmdName(szCommand);
		VERIFY_OK(pApplication->AddCommand(bszCmdString, bszMethod, 1, m_dwCookie, &bRet));
		if (bFirstTime == VARIANT_TRUE)
		{
			VERIFY_OK(pApplication->
				AddCommandBarButton(dsGlyph, bszCmdName, m_dwCookie));
			CComBSTR bszKeystroke("ALT+Z");
			CComBSTR bszEditor("Main");
			VERIFY(SUCCEEDED(pApplication->
				AddKeyBinding(bszKeystroke, bszCmdName, bszEditor)));

		}
	}
	{
		szCommand = _T("ViewAllComments");
		strCmdString.LoadString(IDS_CMD_VIEWALLCOMMENTS);
		strCmdString = szCommand + strCmdString;
		CComBSTR bszCmdString(strCmdString);
		CComBSTR bszMethod(_T("ViewAllComments"));
		CComBSTR bszCmdName(szCommand);
		VERIFY_OK(pApplication->AddCommand(bszCmdString, bszMethod, 2, m_dwCookie, &bRet));
		if (bFirstTime == VARIANT_TRUE)
		{
			VERIFY_OK(pApplication->
				AddCommandBarButton(dsGlyph, bszCmdName, m_dwCookie));
			CComBSTR bszKeystroke("ALT+A");
			CComBSTR bszEditor("Main");
			VERIFY(SUCCEEDED(pApplication->
				AddKeyBinding(bszKeystroke, bszCmdName, bszEditor)));

		}
	}
	{
		szCommand = _T("CommentClear");
		strCmdString.LoadString(IDS_CMD_COMMENTCLEAR);
		strCmdString = szCommand + strCmdString;
		CComBSTR bszCmdString(strCmdString);
		CComBSTR bszMethod(_T("CommentClear"));
		CComBSTR bszCmdName(szCommand);
		VERIFY_OK(pApplication->AddCommand(bszCmdString, bszMethod, 3, m_dwCookie, &bRet));
		if (bFirstTime == VARIANT_TRUE)
		{
			VERIFY_OK(pApplication->
				AddCommandBarButton(dsGlyph, bszCmdName, m_dwCookie));
			CComBSTR bszKeystroke("ALT+C");
			CComBSTR bszEditor("Main");
			VERIFY(SUCCEEDED(pApplication->
				AddKeyBinding(bszKeystroke, bszCmdName, bszEditor)));

		}
	}
	{
		szCommand = _T("CommentDelete");
		strCmdString.LoadString(IDS_CMD_COMMENTDELETE);
		strCmdString = szCommand + strCmdString;
		CComBSTR bszCmdString(strCmdString);
		CComBSTR bszMethod(_T("CommentDelete"));
		CComBSTR bszCmdName(szCommand);
		VERIFY_OK(pApplication->AddCommand(bszCmdString, bszMethod, 4, m_dwCookie, &bRet));
		if (bFirstTime == VARIANT_TRUE)
		{
			VERIFY_OK(pApplication->
				AddCommandBarButton(dsGlyph, bszCmdName, m_dwCookie));
			CComBSTR bszKeystroke("ALT+D");
			CComBSTR bszEditor("Main");
			VERIFY(SUCCEEDED(pApplication->
				AddKeyBinding(bszKeystroke, bszCmdName, bszEditor)));

		}
	}
	{
		szCommand = _T("UpperCase");
		strCmdString.LoadString(IDS_CMD_UPPERCASE);
		strCmdString = szCommand + strCmdString;
		CComBSTR bszCmdString(strCmdString);
		CComBSTR bszMethod(_T("UpperCase"));
		CComBSTR bszCmdName(szCommand);
		VERIFY_OK(pApplication->AddCommand(bszCmdString, bszMethod, 5, m_dwCookie, &bRet));
		if (bFirstTime == VARIANT_TRUE)
		{
			VERIFY_OK(pApplication->
				AddCommandBarButton(dsGlyph, bszCmdName, m_dwCookie));
		}
	}
	{
		szCommand = _T("LowerCase");
		strCmdString.LoadString(IDS_CMD_LOWERCASE);
		strCmdString = szCommand + strCmdString;
		CComBSTR bszCmdString(strCmdString);
		CComBSTR bszMethod(_T("LowerCase"));
		CComBSTR bszCmdName(szCommand);
		VERIFY_OK(pApplication->AddCommand(bszCmdString, bszMethod, 6, m_dwCookie, &bRet));
		if (bFirstTime == VARIANT_TRUE)
		{
			VERIFY_OK(pApplication->
				AddCommandBarButton(dsGlyph, bszCmdName, m_dwCookie));
		}
	}
	{
		szCommand = _T("Capitalize");
		strCmdString.LoadString(IDS_CMD_CAPITALIZE);
		strCmdString = szCommand + strCmdString;
		CComBSTR bszCmdString(strCmdString);
		CComBSTR bszMethod(_T("Capitalize"));
		CComBSTR bszCmdName(szCommand);
		VERIFY_OK(pApplication->AddCommand(bszCmdString, bszMethod, 7, m_dwCookie, &bRet));
		if (bFirstTime == VARIANT_TRUE)
		{
			VERIFY_OK(pApplication->
				AddCommandBarButton(dsGlyph, bszCmdName, m_dwCookie));
		}
	}
	{
		szCommand = _T("SpaceToTab");
		strCmdString.LoadString(IDS_CMD_SPACETOTAB);
		strCmdString = szCommand + strCmdString;
		CComBSTR bszCmdString(strCmdString);
		CComBSTR bszMethod(_T("SpaceToTab"));
		CComBSTR bszCmdName(szCommand);
		VERIFY_OK(pApplication->AddCommand(bszCmdString, bszMethod, 8, m_dwCookie, &bRet));
		if (bFirstTime == VARIANT_TRUE)
		{
			VERIFY_OK(pApplication->
				AddCommandBarButton(dsGlyph, bszCmdName, m_dwCookie));
		}
	}
	{
		szCommand = _T("AddCodeTemplate");
		strCmdString.LoadString(IDS_CMD_CODETMPLS);
		strCmdString = szCommand + strCmdString;
		CComBSTR bszCmdString(strCmdString);
		CComBSTR bszMethod(_T("AddCodeTemplate"));
		CComBSTR bszCmdName(szCommand);
		VERIFY_OK(pApplication->AddCommand(bszCmdString, bszMethod, 9, m_dwCookie, &bRet));
		if (bFirstTime == VARIANT_TRUE)
		{
			VERIFY_OK(pApplication->
				AddCommandBarButton(dsGlyph, bszCmdName, m_dwCookie));
			CComBSTR bszKeystroke("CTRL+J");
			CComBSTR bszEditor("Main");
			VERIFY(SUCCEEDED(pApplication->
				AddKeyBinding(bszKeystroke, bszCmdName, bszEditor)));
		}
	}
	{
		szCommand = _T("About");
		strCmdString.LoadString(IDS_CMD_ABOUT);
		strCmdString = szCommand + strCmdString;
		CComBSTR bszCmdString(strCmdString);
		CComBSTR bszMethod(_T("About"));
		CComBSTR bszCmdName(szCommand);
		VERIFY_OK(pApplication->AddCommand(bszCmdString, bszMethod, 10, m_dwCookie, &bRet));
		if (bFirstTime == VARIANT_TRUE)
		{
			VERIFY_OK(pApplication->
				AddCommandBarButton(dsGlyph, bszCmdName, m_dwCookie));
		}
	}
	if (bRet == VARIANT_FALSE)
	{
		// AddCommand failed because a command with this name already
		//  exists.  You may try adding your command under a different name.
		//  Or, you can fail to load as we will do here.
		*OnConnection = VARIANT_FALSE;
		return S_OK;
	}

	*OnConnection = VARIANT_TRUE;
	return S_OK;
}

// This is called on shut-down, and also when the user unloads the add-in
STDMETHODIMP CDSAddIn::OnDisconnection(VARIANT_BOOL bLastTime)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	m_pCommands->UnadviseFromEvents();
	m_pCommands->Release();
	m_pCommands = NULL;

	return S_OK;
}
