/////////////////////////////////////////////////////////////////////////////
//
//	Project		:	Active Commentary v1.0 (Visual C++ Dev Studio AddIn)
//	File		:	CommentaryDlg.h
//	Description	:	Implementation of List All Comments dialog
//					(CListDlg class)
//
//	Created and modified by Andrey Levin
//	mail to: Levin@artis-inventum.de
/////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "ActiveComments.h"
#include "Commands.h"
#include "ListDlg.h"
#include "CommentaryDlg.h"
#include <comdef.h>


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CListDlg dialog


CListDlg::CListDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CListDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CListDlg)
	//}}AFX_DATA_INIT
}


void CListDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CListDlg)
	DDX_Control(pDX, IDC_OPENDOC, m_OpenDoc);
	DDX_Control(pDX, IDC_GOTO, m_Goto);
	DDX_Control(pDX, IDC_EXECUTE, m_Execute);
	DDX_Control(pDX, IDC_EDIT, m_Edit);
	DDX_Control(pDX, IDC_DELETE, m_Delete);
	DDX_Control(pDX, IDC_CLEAR, m_Clear);
	DDX_Control(pDX, IDC_PROJECTTREE, m_ProjectTree);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CListDlg, CDialog)
	//{{AFX_MSG_MAP(CListDlg)
	ON_BN_CLICKED(IDC_DELETE, OnDelete)
	ON_BN_CLICKED(IDC_CLEAR, OnClear)
	ON_BN_CLICKED(IDC_GOTO, OnGoto)
	ON_BN_CLICKED(IDC_EXECUTE, OnExecute)
	ON_BN_CLICKED(IDC_EDIT, OnEdit)
	ON_NOTIFY(NM_DBLCLK, IDC_PROJECTTREE, OnDblclkProjecttree)
	ON_NOTIFY(TVN_SELCHANGED, IDC_PROJECTTREE, OnSelchangedProjecttree)
	ON_BN_CLICKED(IDC_OPENDOC, OnOpendoc)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CListDlg message handlers

BOOL CListDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	POSITION pos1 = m_pProjectMap->GetStartPosition(), pos2;
	CString szKey;
	HTREEITEM hItem;
	
	TVINSERTSTRUCT tvInsert;
	
	m_pDoc = NULL;
	m_pSel = NULL;	

	if(GetCurrentDocument(m_pApp, m_pDoc))
		GetCurrentSelection(m_pDoc, m_pSel);
	else
	{
		AfxMessageBox("Error: Cannot get pointer to current document");
		return FALSE;
	}
	
	m_ProjectTree.DeleteAllItems();
	CComBSTR bstr;
	m_pDoc->get_Name(&bstr);

	CBitmap bmp1,bmp2;
	m_ImageList.Create(16,16,ILC_COLOR,2,0);
	bmp1.LoadBitmap(IDB_DOCUMENT);
	m_ImageList.Add(&bmp1,RGB(255, 255, 155));
	bmp2.LoadBitmap(IDB_COMMENT);
	m_ImageList.Add(&bmp2,RGB(0, 0, 0));
	m_ProjectTree.SetImageList(&m_ImageList,TVSIL_NORMAL);	
	try
	{
		for( pos1 = m_pProjectMap->GetStartPosition(); pos1 != NULL; )
		{
			m_pProjectMap->GetNextAssoc( pos1, szKey, (CObject*&)m_pDocumentMap);
			
			tvInsert.hParent = NULL;
			tvInsert.hInsertAfter = NULL;
			tvInsert.item.mask = TVIF_TEXT;
			tvInsert.item.pszText = (LPTSTR)((LPCTSTR)szKey);
			HTREEITEM hDocName = m_ProjectTree.InsertItem(&tvInsert);
			m_ProjectTree.SetItemImage(hDocName,0,0);
			if(szKey.Compare((CString)bstr) == 0)
				hItem = hDocName;
			for( pos2 = m_pDocumentMap->GetStartPosition(); pos2 != NULL; )
			{
				m_pDocumentMap->GetNextAssoc( pos2, szKey, (CObject*&)m_pComment);
				HTREEITEM hTitle = m_ProjectTree.InsertItem(TVIF_TEXT,
					(LPTSTR)((LPCTSTR)szKey), 0, 0, 0, 0, 0, hDocName, NULL);
				m_ProjectTree.SetItemImage(hTitle ,1,1);
			}
		}
	}
	catch(...)
	{
		TRACE("\n************ Error init ListDlg ************\n");
	}
	
	m_ProjectTree.SetFocus();	
	if(hItem)
	{
		m_ProjectTree.Expand(hItem,TVE_EXPAND);
		m_ProjectTree.SelectItem(hItem);
	}


	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CListDlg::OnDelete() 
{
	CString Title;
	_variant_t DelType,SelType;
	VARIANT_BOOL ret;
	CComBSTR bstr;
	
	HTREEITEM hSelectedItem = m_ProjectTree.GetSelectedItem();

	DWORD dwRet = FindComment(hSelectedItem, MODE_DELETE);
	if(dwRet != ITEM_SUCCESFULLYFINDED)
	{
		if(dwRet == ITEM_NOTATITLE)
		{
			Title = m_ProjectTree.GetItemText(hSelectedItem);
			m_pProjectMap->RemoveKey(Title);
			m_ProjectTree.DeleteItem(hSelectedItem);
			if(m_pDocumentMap!=NULL)
			{
				POSITION pos;
				try
				{
					for( pos = m_pDocumentMap->GetStartPosition(); pos != NULL; )
					{
						m_pDocumentMap->GetNextAssoc( pos, Title, (CObject*&)m_pComment);
						delete m_pComment;
						m_pComment = NULL;
					}
					m_pDocumentMap->RemoveAll();
					delete m_pDocumentMap;
					m_pDocumentMap = NULL;
				}
				catch(...)
				{
					TRACE("\n************ DELETE ERROR ************\n");
				}
			}
		}
		return;
	}
	Title = m_ProjectTree.GetItemText(hSelectedItem);
	m_pDocumentMap->RemoveKey(Title);
	delete m_pComment;

	Title = "[[->" + Title + "]]";
	DelType.iVal = dsFirstColumn;
	DelType.vt = VT_I2;
	SelType.iVal = dsMove;
	SelType.vt = VT_I2;
	m_pSel->StartOfDocument(SelType);
	SelType.iVal = NULL;
	bstr = Title.AllocSysString();
	m_pSel->FindText(bstr,SelType,&ret);
	DelType.iVal = 1;
	if(ret)
		m_pSel->Delete(DelType);

	m_ProjectTree.DeleteItem(hSelectedItem);
}

void CListDlg::OnClear() 
{
	CString Title;
	_variant_t DelType,SelType;
	VARIANT_BOOL ret;
	CComBSTR bstr;

	HTREEITEM hSelectedItem = m_ProjectTree.GetSelectedItem();

	DWORD dwRet = FindComment(hSelectedItem, MODE_CLEAR);
	if(dwRet != ITEM_SUCCESFULLYFINDED)
		return;
	
	Title = m_ProjectTree.GetItemText(hSelectedItem);
	m_pDocumentMap->RemoveKey(Title);
	delete m_pComment;

	DelType.iVal = dsFirstColumn;
	DelType.vt = VT_I2;
	SelType.iVal = dsMove;
	SelType.vt = VT_I2;
	CString s = "[[->" + Title + "]]";
	m_pSel->StartOfDocument(SelType);
	SelType.iVal = NULL;
	bstr = s.AllocSysString();
	m_pSel->FindText(bstr,SelType,&ret);
	DelType.iVal = 1;
	if(ret)
		m_pSel->Delete(DelType);
	bstr = Title.AllocSysString();
	m_pSel->put_Text(bstr);
	s.ReleaseBuffer();
	Title.ReleaseBuffer();

	m_ProjectTree.DeleteItem(hSelectedItem);
}

void CListDlg::OnGoto() 
{
	_variant_t SelType;
	VARIANT_BOOL ret;
	CComBSTR bstr;

	HTREEITEM hSelectedItem = m_ProjectTree.GetSelectedItem();

	DWORD dwRet = FindComment(hSelectedItem, MODE_GOTO);
	if(dwRet != ITEM_SUCCESFULLYFINDED)
		return;
	
	SelType.iVal = dsMove;
	SelType.vt = VT_I2;
	CString s = "[[->" + m_ProjectTree.GetItemText(hSelectedItem)+ "]]";
	m_pSel->StartOfDocument(SelType);
	SelType.iVal = NULL;
	bstr = s.AllocSysString();
	m_pSel->FindText(bstr,SelType,&ret);
	s.ReleaseBuffer();

}

void CListDlg::OnExecute() 
{
	HTREEITEM hSelectedItem = m_ProjectTree.GetSelectedItem();

	DWORD dwRet = FindComment(hSelectedItem, MODE_EXECUTE);
	if(dwRet != ITEM_SUCCESFULLYFINDED)
		return;
	else	
		ExecuteLink(m_pComment);
	
}

void CListDlg::OnEdit() 
{
	CString Title,oldTitle, sAddPrComm ="";
	_variant_t DelType,SelType;
	CComBSTR bstr;
	DWORD dwRet;

	HTREEITEM hSelectedItem = m_ProjectTree.GetSelectedItem();

	dwRet = FindComment(hSelectedItem, MODE_EDIT);
	if(dwRet != ITEM_SUCCESFULLYFINDED)
		return;
	
	long row;
	m_pSel->get_CurrentLine(&row);
	CCommentaryDlg dlg;
	Title = m_ProjectTree.GetItemText(hSelectedItem);
	dlg.m_szTitle = Title;
	dlg.m_szLink = m_pComment->m_szLink;
	dlg.m_szProtocol = m_pComment->m_szProtocol;
	oldTitle = Title;
	dlg.m_szInitPath = GetFilePath(GetProjectName()) + "\\";
	dlg.m_szDocumentName = m_pComment->m_szDocumentName;
	dlg.m_pSel = m_pSel;
	dlg.m_pMap = m_pDocumentMap;
	dlg.m_nTime = m_pComment->m_nTimeOfCreate;
	dlg.m_pStrArray = m_pStrArray;
	if(dlg.DoModal() != IDOK)
		return;
	Title = dlg.m_szTitle;
	
	m_pComment->m_szLink = dlg.m_szLink;
	m_pComment->m_szProtocol = dlg.m_szProtocol;
	if(oldTitle.Compare(Title) == 0)
	   return;
	else
	{
		HTREEITEM hParentItem = m_ProjectTree.GetParentItem(hSelectedItem);		
		m_ProjectTree.DeleteItem(hSelectedItem);
		m_ProjectTree.InsertItem(TVIF_TEXT,(LPTSTR)((LPCTSTR)Title), 0, 0, 0, 0, 0, hParentItem, NULL);
		m_ProjectTree.SortChildren(hParentItem);
		m_pDocumentMap->RemoveKey(oldTitle);
		m_pDocumentMap->SetAt(Title,m_pComment);
	}

	if(!oldTitle.IsEmpty())
		if(SelectComment(oldTitle,m_pSel))
		{
			SelType.iVal = 1;
			SelType.vt = VT_I2;
			if(IsSelectionInTheCommentBlock(m_pSel) == SEARCH_ISNO)
				sAddPrComm = "// ";
			m_pSel->Delete(SelType);
		}
	
	
	CString s = sAddPrComm + "[[->" + Title + "]]";
	bstr = s.AllocSysString();
	m_pSel->put_Text(bstr);
	s.ReleaseBuffer();
	
}

////////////////////////////////////////////////////////////
//	Find comment by selected item. Result : set pointers 
//	(m_pDocumentMap and m_pComment) on finded comment
DWORD CListDlg::FindComment(HTREEITEM hItem, DWORD forMode) 
{
	CString Title;
	_variant_t DelType,SelType;
	CComBSTR bstr;

	if(hItem == NULL)
		return ITEM_EMPTY;

	HTREEITEM hParentItem = m_ProjectTree.GetParentItem(hItem);
	if(hParentItem == NULL)
		return ITEM_NOTATITLE;

	Title = m_ProjectTree.GetItemText(hParentItem);
	if(!m_pProjectMap->Lookup(Title,(CObject*&) m_pDocumentMap))
	{
		TRACE("ListDlg: Find link to Document");
		if(AfxMessageBox("Cannot find this Document. Remove from base?", MB_YESNO) == IDYES)
		{		
			m_ProjectTree.DeleteItem(hParentItem);
			m_pProjectMap->RemoveKey(Title);
		}
		return ITEM_CANNOTFINDDOC;
	}
	
	Title = m_ProjectTree.GetItemText(hItem);
	if(!m_pDocumentMap->Lookup(Title,(CObject*&) m_pComment))
	{
		TRACE("\n************ ListDlg: Delete Error ************\n");
		if(AfxMessageBox("Cannot find this Commentary. Remove from base?", MB_YESNO) == IDYES)
		{		
			m_ProjectTree.DeleteItem(hItem);
			m_pDocumentMap->RemoveKey(Title);
		}
		return ITEM_FINDFAILURE;
	}

	return ITEM_SUCCESFULLYFINDED;
}


void CListDlg::OnDblclkProjecttree(NMHDR* pNMHDR, LRESULT* pResult) 
{
	CString DocName;
	HTREEITEM hItem = m_ProjectTree.GetSelectedItem();
	HTREEITEM hParent = m_ProjectTree.GetParentItem(hItem);
	if(hParent != NULL)
		hItem = hParent;

	DocName = m_ProjectTree.GetItemText(hItem);

	if(!OpenDocument(m_pApp,DocName))
		AfxMessageBox((CString)("Cannot open document: "+DocName));
	else
	{
		OnSelchangedProjecttree(pNMHDR, pResult);
		m_ProjectTree.Expand(hItem,TVE_EXPAND);
	}

	if(m_pSel)
		m_pSel->Release();
	if(m_pDoc)
		m_pDoc->Release();
	m_pDoc = NULL;
	m_pSel = NULL;
	
	if(GetCurrentDocument(m_pApp, m_pDoc))
		GetCurrentSelection(m_pDoc, m_pSel);
	else
		AfxMessageBox("Error: Cannot get pointer to current document");

	if(hParent != NULL)
		OnEdit();

	*pResult = 0;
}

void CListDlg::OnSelchangedProjecttree(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_TREEVIEW* pNMTreeView = (NM_TREEVIEW*)pNMHDR;
	
	CString DocName;
	HTREEITEM hItem = m_ProjectTree.GetSelectedItem();
	HTREEITEM hParent = m_ProjectTree.GetParentItem(hItem);
	if(hParent != NULL)
		hItem = hParent;

	DocName = m_ProjectTree.GetItemText(hItem);

	CComBSTR bstr;
	m_pDoc->get_Name(&bstr);
	if(DocName.Compare((CString)bstr) == 0)
	{
		if(hItem == hParent)
		{
			m_Edit.EnableWindow(TRUE);
			m_Clear.EnableWindow(TRUE);
			m_Goto.EnableWindow(TRUE);
			m_Execute.EnableWindow(TRUE);
			m_OpenDoc.EnableWindow(FALSE);
		}
		else
		{
			m_Edit.EnableWindow(FALSE);
			m_Clear.EnableWindow(FALSE);
			m_Goto.EnableWindow(FALSE);
			m_Execute.EnableWindow(FALSE);
			m_OpenDoc.EnableWindow(TRUE);
		}
	}
	else
	{
		m_Edit.EnableWindow(FALSE);
		m_Clear.EnableWindow(FALSE);
		m_Goto.EnableWindow(FALSE);
		if(hItem == hParent)
		{
			m_Execute.EnableWindow(TRUE);
			m_OpenDoc.EnableWindow(FALSE);
		}
		else
		{
			m_Execute.EnableWindow(FALSE);
			m_OpenDoc.EnableWindow(TRUE);
		}
	}
	
	*pResult = 0;
}

BOOL CListDlg::DestroyWindow() 
{
	if(m_pSel)
		m_pSel->Release();
	if(m_pDoc)
		m_pDoc->Release();
	m_pDoc = NULL;
	m_pSel = NULL;	

	for (int i = 0; i < m_ImageList.GetImageCount(); i++)
		m_ImageList.Remove(i);

	return CDialog::DestroyWindow();
}

void CListDlg::OnOK() 
{
	if(m_pSel)
		m_pSel->Release();
	if(m_pDoc)
		m_pDoc->Release();
	m_pDoc = NULL;
	m_pSel = NULL;	

	CDialog::OnOK();
}

void CListDlg::OnOpendoc() 
{
	CString DocName;
	HTREEITEM hItem = m_ProjectTree.GetSelectedItem();
	HTREEITEM hParent = m_ProjectTree.GetParentItem(hItem);
	if(hParent != NULL)
		return;

	DocName = m_ProjectTree.GetItemText(hItem);

	if(!OpenDocument(m_pApp,DocName))
		AfxMessageBox((CString)("Cannot open document: "+DocName));
	else
		m_ProjectTree.Expand(hItem,TVE_EXPAND);

	if(m_pSel)
		m_pSel->Release();
	if(m_pDoc)
		m_pDoc->Release();
	m_pDoc = NULL;
	m_pSel = NULL;
	
	if(GetCurrentDocument(m_pApp, m_pDoc))
		GetCurrentSelection(m_pDoc, m_pSel);
	else
		AfxMessageBox("Error: Cannot get pointer to current document");
}
