// MCIRecorder.cpp: implementation of the CMCIRecorder class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "ActiveComments.h"
#include "MCIRecorder.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CMCIRecorder::CMCIRecorder()
{
	m_iDeviceID		=	0;
	m_bOverwrite	=	FALSE;
	m_bModified		=	FALSE;
	m_iState		=	MPS_CLOSED;
}

CMCIRecorder::~CMCIRecorder()
{
	Close();
}

/////////////////////////////////////////////////////////////////
//			Create new wave-file
/////////////////////////////////////////////////////////////////
DWORD	CMCIRecorder::New(DWORD dwFormat) // only new sound !!!!!
{
	Close();

	m_szFileName = "noname.wav";
	mciOpenParms.lpstrDeviceType = "waveaudio";
	mciOpenParms.lpstrElementName = "";
	mciSaveParms.lpfilename = m_szFileName;
	if (m_dwReturn = mciSendCommand(0, MCI_OPEN,
						MCI_OPEN_ELEMENT | MCI_OPEN_TYPE, 
						(DWORD)(LPVOID) &mciOpenParms))
	{
		CHAR sErr[250];
		if(mciGetErrorString(m_dwReturn, sErr, 250))
		{
			TRACE("\n************ (On New) MCI error is: %s ************\n", sErr);
		}
		m_iState = MPS_CLOSED;
		return m_dwReturn;
    }
	m_iState = MPS_OPENED;
	m_iDeviceID	= mciOpenParms.wDeviceID;
	
	SetWaveFormat(dwFormat);	
	
	m_iCountTracks	=	1;
	m_iCurrentTrack	=	1;
	return m_dwReturn;
}

/////////////////////////////////////////////////////////////////
//			Open media file with DeviceName
/////////////////////////////////////////////////////////////////
DWORD	CMCIRecorder::OpenDevice(LPCSTR FileName, LPCSTR Device)
{
	DWORD dwFlags = MCI_OPEN_ELEMENT;
	Close();

	m_szFileName = FileName;
	mciOpenParms.lpstrElementName = m_szFileName;
	mciOpenParms.lpstrDeviceType = NULL;
	
	if(!strcmp(Device,"cdaudio"))
	{
		mciOpenParms.lpstrDeviceType = "cdaudio";
		dwFlags = MCI_OPEN_ELEMENT | MCI_OPEN_TYPE;
	}
	else	
		mciSaveParms.lpfilename = m_szFileName;
	
	if (m_dwReturn = mciSendCommand(0, MCI_OPEN, dwFlags, 
									(DWORD)(LPVOID) &mciOpenParms))  
	{
		CHAR sErr[250];
		if(mciGetErrorString(m_dwReturn, sErr, 250))
		{
			TRACE("\n************ (On Open) MCI error is: %s ************\n", sErr);
		}
		m_iState	=	MPS_CLOSED;
		return m_dwReturn;
    }

    // The device opened successfully; get the device ID.
    m_iDeviceID	=	mciOpenParms.wDeviceID;
	m_iState	=	MPS_OPENED;

	m_iCountTracks	=	GetCountTracks();
	m_iCurrentTrack	=	1;
	
	SetDeviceInfo(MCI_SET_TIME_FORMAT,MCI_FORMAT_MILLISECONDS);
	
	return m_dwReturn;
}

/////////////////////////////////////////////////////////////////
//			Close opened device
/////////////////////////////////////////////////////////////////
DWORD	CMCIRecorder::Close()
{
	if(GetState() != MPS_CLOSED)
	{
		m_dwReturn	=	mciSendCommand(m_iDeviceID, MCI_CLOSE, 0, NULL);
		m_iState	=	MPS_STOPED;
		m_iCurrentTrack	=	0;
		m_iCountTracks	=	0;
		return m_dwReturn;
	}
	else
		return 0; // yet closed
}

		 
/////////////////////////////////////////////////////////////////
//			Saving wave file
/////////////////////////////////////////////////////////////////
DWORD	CMCIRecorder::Save(LPCSTR FileName)
{

	if(!FileName)
		m_szFileName = "noname.wav";
	else
		m_szFileName = FileName;

	mciSaveParms.lpfilename = m_szFileName;
							  
    if (m_dwReturn = mciSendCommand(m_iDeviceID, MCI_SAVE,
        MCI_SAVE_FILE | MCI_WAIT, (DWORD)(LPVOID) &mciSaveParms))
    {
        mciSendCommand(m_iDeviceID, MCI_CLOSE, 0, NULL);
		CHAR sErr[250];
		if(mciGetErrorString(m_dwReturn, sErr, 250))
		{
			TRACE("\n************ (On Save) MCI error is: %s ************\n", sErr);
		}
		m_iState	=	MPS_STOPED;
    }
	else
		m_iState	=	MPS_STOPED;
	m_bModified	=	FALSE;
	return (m_dwReturn);
}

/////////////////////////////////////////////////////////////////
//			Play wave or video file
/////////////////////////////////////////////////////////////////
DWORD	CMCIRecorder::Play(DWORD From, DWORD To)
{
	DWORD Flags;

	if(!To)
		Flags = MCI_NOTIFY | MCI_FROM ;
	else
		Flags = MCI_NOTIFY | MCI_FROM | MCI_TO;
	if(!From)
	{
		DWORD StartPos = GetTrackPosition(m_iCurrentTrack);
		mciPlayParms.dwFrom = StartPos;
	}
	else
	{
		mciPlayParms.dwFrom = From;
		mciPlayParms.dwTo = To;
	}

    if (m_dwReturn = mciSendCommand(m_iDeviceID, MCI_PLAY,
									Flags, (DWORD)(LPVOID) &mciPlayParms))
    {
        mciSendCommand(m_iDeviceID, MCI_CLOSE, 0, NULL);
		CHAR sErr[250];
		if(mciGetErrorString(m_dwReturn, sErr, 250))
		{
			TRACE("\n************ (On Play) MCI error is: %s ************\n", sErr);
		}
		m_iState	=	MPS_STOPED;
    }
	else
		m_iState	=	MPS_PLAYING;
	return (m_dwReturn);
}

/////////////////////////////////////////////////////////////////
//			Recording wave
/////////////////////////////////////////////////////////////////
DWORD	CMCIRecorder::Record()
{
	if(m_bOverwrite)
	   m_dwReturn = mciSendCommand(m_iDeviceID, MCI_RECORD, 
        MCI_RECORD_OVERWRITE, (DWORD)(LPVOID) &mciRecordParms);
	else
	   m_dwReturn = mciSendCommand(m_iDeviceID, MCI_RECORD, 
        MCI_RECORD_INSERT, (DWORD)(LPVOID) &mciRecordParms);
		
	m_iState	=	MPS_RECORDING;
	if (m_dwReturn)
    {
        mciSendCommand(m_iDeviceID, MCI_CLOSE, 0, NULL);
		CHAR sErr[250];
		if(mciGetErrorString(m_dwReturn, sErr, 250))
		{
			TRACE("\n************ (On Record) MCI error is: %s ************\n", sErr); 
		}
		m_iState	=	MPS_CLOSED;
	}
	m_bModified	=	TRUE;
	return (m_dwReturn);
}

/////////////////////////////////////////////////////////////////
//				 play/record
/////////////////////////////////////////////////////////////////
DWORD	CMCIRecorder::Stop()
{
	if (m_dwReturn = mciSendCommand(m_iDeviceID, MCI_STOP, 0, NULL))
		mciSendCommand(m_iDeviceID, MCI_CLOSE, 0, NULL);
	m_iState	=	MPS_STOPED;
	return m_dwReturn;
}

/////////////////////////////////////////////////////////////////
//			Pause play
/////////////////////////////////////////////////////////////////
DWORD	CMCIRecorder::Pause()
{
	if((GetState() != MPS_PLAYING)&&(GetState() != MPS_RECORDING))
		return 0;

	if(GetState() == MPS_PAUSING)
		return Resume();

	m_iState	=	MPS_PAUSING;
	mciGenParms.dwCallback = NULL;
	m_dwReturn = mciSendCommand( m_iDeviceID, MCI_PAUSE, MCI_WAIT, (DWORD) &mciGenParms);
	
	return m_dwReturn;
}

/////////////////////////////////////////////////////////////////
//			Resume play
/////////////////////////////////////////////////////////////////
DWORD	CMCIRecorder::Resume()
{
	if(GetState() == MPS_CLOSED)
		return 0;

	if(m_dwReturn = mciSendCommand(m_iDeviceID, MCI_RESUME,0, (DWORD) &mciGenParms))
	{
		CHAR sErr[250];
		if(mciGetErrorString(m_dwReturn, sErr, 250))
			TRACE("\n************ (On Resume) MCI error is: %s ************\n", sErr);
	}
	m_iState	=	MPS_PLAYING;
	return m_dwReturn;
}

////////////////////////////////////////////////////////////
//					Set MCIDevice Info
////////////////////////////////////////////////////////////
DWORD	CMCIRecorder::SetDeviceInfo(DWORD SetFlags, DWORD Parameter)
{
	
	switch(SetFlags)
	{
		case MCI_SET_TIME_FORMAT : 	mciSet.dwCallback = 0;
									mciSet.dwTimeFormat = Parameter;
									break;
					  /*int TheLength;
					  TimeFormat = tfHMS;
					  TheLength = MediaPlayer1->Length; 
					  hours = IntToStr(LOBYTE(LOWORD(TheLength)));
					  minutes = IntToStr(HIBYTE(LOWORD(TheLength)));
					  seconds = IntToStr(LOBYTE(HIWORD(TheLength)));*/
		case MCI_SET_AUDIO	:	 	mciSet.dwCallback = 0;
									mciSet.dwAudio = Parameter;
									break;
	}

	if (m_dwReturn = mciSendCommand(m_iDeviceID, MCI_SET, SetFlags, (DWORD)(LPSTR)&mciSet))
	{
		char sErr[250];
		if(mciGetErrorString(m_dwReturn, sErr, 250))
		{
			TRACE("\n************ (On SetDeviceInfo) MCI error is: %s ************\n", sErr);
		}
	}

	return m_dwReturn;
}

////////////////////////////////////////////////////////////
//					Set Wave Format
////////////////////////////////////////////////////////////
DWORD CMCIRecorder::SetWaveFormat(DWORD dwFormat)
{
	DWORD dwFlags = MCI_WAVE_SET_FORMATTAG | 
					MCI_WAVE_SET_CHANNELS | 
					MCI_WAVE_SET_BLOCKALIGN | 
					MCI_WAVE_SET_BITSPERSAMPLE | 
					MCI_WAVE_SET_SAMPLESPERSEC;
    
	mciWaveParms.wFormatTag = WAVE_FORMAT_PCM;
	mciWaveParms.wInput = m_dwDevice;
	mciWaveParms.wOutput = m_dwDevice;
	mciWaveParms.dwTimeFormat = MCI_FORMAT_MILLISECONDS;
	mciWaveParms.dwCallback = NULL;
    switch (dwFormat) {
    case WAVE_FORMAT_1M08:
        mciWaveParms.nChannels = 1;
        mciWaveParms.nSamplesPerSec = 11025;
		mciWaveParms.wBitsPerSample = 8;
        break;
    case WAVE_FORMAT_1M16:
        mciWaveParms.nChannels = 1;
        mciWaveParms.nSamplesPerSec = 11025;
        mciWaveParms.wBitsPerSample = 16;
        break;
    case WAVE_FORMAT_1S08:
        mciWaveParms.nChannels = 2;
        mciWaveParms.nSamplesPerSec = 11025;
        mciWaveParms.wBitsPerSample = 8;
        break;
    case WAVE_FORMAT_1S16:
        mciWaveParms.nChannels = 2;
        mciWaveParms.nSamplesPerSec = 11025;
        mciWaveParms.wBitsPerSample = 16;
        break;
    case WAVE_FORMAT_2M08:
        mciWaveParms.nChannels = 1;
        mciWaveParms.nSamplesPerSec = 22050;
        mciWaveParms.wBitsPerSample = 8;
        break;
    case WAVE_FORMAT_2M16:
        mciWaveParms.nChannels = 1;
        mciWaveParms.nSamplesPerSec = 22050;
        mciWaveParms.wBitsPerSample = 16;
        break;
    case WAVE_FORMAT_2S08:
        mciWaveParms.nChannels = 2;
        mciWaveParms.nSamplesPerSec = 22050;
        mciWaveParms.wBitsPerSample = 8;
        break;
    case WAVE_FORMAT_2S16:
        mciWaveParms.nChannels = 2;
        mciWaveParms.nSamplesPerSec = 22050;
        mciWaveParms.wBitsPerSample = 16;
        break;
    case WAVE_FORMAT_4M08:
        mciWaveParms.nChannels = 1;
        mciWaveParms.nSamplesPerSec = 44100;
        mciWaveParms.wBitsPerSample = 8;
        break;
    case WAVE_FORMAT_4M16:
        mciWaveParms.nChannels = 1;
        mciWaveParms.nSamplesPerSec = 44100;
        mciWaveParms.wBitsPerSample = 16;
        break;
    case WAVE_FORMAT_4S08:
        mciWaveParms.nChannels = 2;
        mciWaveParms.nSamplesPerSec = 44100;
        mciWaveParms.wBitsPerSample = 8;
        break;
    case WAVE_FORMAT_4S16:
        mciWaveParms.nChannels = 2;
        mciWaveParms.nSamplesPerSec = 44100;
        mciWaveParms.wBitsPerSample = 16;
        break;

    }

	mciWaveParms.nAvgBytesPerSec = mciWaveParms.nSamplesPerSec;
	mciWaveParms.nBlockAlign = mciWaveParms.wBitsPerSample / 8;

	if (m_dwReturn = mciSendCommand(m_iDeviceID, MCI_SET, 
			dwFlags, (DWORD)(LPSTR)&mciWaveParms))
	{
		char sErr[250];
		if(mciGetErrorString(m_dwReturn, sErr, 250))
		{
			TRACE("\n************ (On SetWaveFormat) MCI error is: %s ************\n", sErr);
		}
	}

	return m_dwReturn;
}

///////////////////////////////////////////////////////////
//					Get Count Tracks
////////////////////////////////////////////////////////////
DWORD  CMCIRecorder::GetCountTracks()
{
	if(GetState() == MPS_CLOSED)
		return 0;

	mciStatus.dwItem = MCI_STATUS_NUMBER_OF_TRACKS ;
	if (m_dwReturn = mciSendCommand(m_iDeviceID, MCI_STATUS, MCI_STATUS_ITEM , (DWORD)(LPSTR)&mciStatus))
	{
		char sErr[250];
		if(mciGetErrorString(m_dwReturn, sErr, 250))
		{
			TRACE("\n************ (On GetCountTracks) MCI error is: %s ************\n", sErr);
		}
	}
	m_iCountTracks = mciStatus.dwReturn;
	return m_iCountTracks;  
}


////////////////////////////////////////////////////////////
//					Get Current Track Position
////////////////////////////////////////////////////////////
DWORD CMCIRecorder::GetCurrentPosition()
{
	mciStatus.dwItem = MCI_STATUS_POSITION;
	if (m_dwReturn = mciSendCommand(m_iDeviceID, MCI_STATUS, MCI_STATUS_ITEM , (DWORD)(LPSTR)&mciStatus))
	{
		char sErr[250];
		if(mciGetErrorString(m_dwReturn, sErr, 250))
		{
			TRACE("\n************ (On GetCurrentPosition) MCI error is: %s ************\n", sErr);
		}
		return 0;
	}
	
	return mciStatus.dwReturn;
}

////////////////////////////////////////////////////////////
//					Set Current Position
////////////////////////////////////////////////////////////
DWORD	CMCIRecorder::SetPosition(DWORD	seekTo)
{
	//MCIERROR mciSendCommand(MCIDEVICEID wDeviceID,   MCI_SEEK,   DWORD dwFlags,   (DWORD) (LPMCI_SEEK_PARMS) lpSeek);
	mciSeek.dwCallback = 0;
	mciSeek.dwTo = seekTo;
	if (m_dwReturn = mciSendCommand(m_iDeviceID, MCI_SEEK, MCI_TO, (DWORD)(LPSTR)&mciSeek))
	{
		char sErr[250];
		if(mciGetErrorString(m_dwReturn, sErr, 250))
		{
			TRACE("\n************ (On SetPosition) MCI error is: %s ************\n", sErr);
		}
	}
	return m_dwReturn;
}

////////////////////////////////////////////////////////////
//					Get Track Position
////////////////////////////////////////////////////////////
DWORD  CMCIRecorder::GetTrackPosition(UINT TrackNum)
{
	if(GetState() == MPS_CLOSED)
		return 0;

	if((TrackNum > GetCountTracks())||(TrackNum  < 1))
		return 0;

	mciStatus.dwItem = MCI_STATUS_POSITION;
	mciStatus.dwTrack = (DWORD)TrackNum;
	if (m_dwReturn = mciSendCommand(m_iDeviceID, MCI_STATUS, 
			MCI_WAIT | MCI_STATUS_ITEM | MCI_TRACK, (DWORD)(LPSTR)&mciStatus))
	{
		char sErr[250];
		if(mciGetErrorString(m_dwReturn, sErr, 250))
		{
			TRACE("\n************ (On GetTrackPosition) MCI error is: %s ************\n", sErr);
		}
	}
	return mciStatus.dwReturn;  
}

////////////////////////////////////////////////////////////
//					Set Current Track
////////////////////////////////////////////////////////////
DWORD	CMCIRecorder::SetCurrectTrack(UINT TrackNum)
{
	DWORD	StartPos;
	CString s;	
	
	s.Format("Can't load track %d: out of range",TrackNum);	

	if(TrackNum < m_iCountTracks)
	{
		StartPos = GetTrackPosition(TrackNum);
		m_iCurrentTrack = TrackNum;
	}
	else
		AfxMessageBox(s, MB_ICONERROR);

//	if(m_iState == MPS_PLAYING)
//		return Play(StartPos,NULL);

	return 0;
}

////////////////////////////////////////////////////////////
//					Get Length of Track
////////////////////////////////////////////////////////////
DWORD  CMCIRecorder::GetTrackLength(UINT TrackNum)
{
	if(GetState() == MPS_CLOSED)
		return 0;

	if((TrackNum > GetCountTracks())||(TrackNum  < 1))
		return 0;
	mciStatus.dwItem = MCI_STATUS_LENGTH;
	mciStatus.dwTrack = (DWORD)TrackNum;
	if (m_dwReturn = mciSendCommand(m_iDeviceID, MCI_STATUS, 
		MCI_WAIT | MCI_STATUS_ITEM | MCI_TRACK, (DWORD)(LPSTR)&mciStatus))
	{
		char sErr[250];
		if(mciGetErrorString(m_dwReturn, sErr, 250))
		{
			TRACE("\n************ (On GetTrackLength) MCI error is: %s ************\n", sErr);
		}
		return 0;
	}

	return mciStatus.dwReturn;
}

// Open wave-file
DWORD CMCIRecorder::Open(LPCSTR FileName)
{
	DWORD dwFlags = MCI_OPEN_ELEMENT;
	Close();

	m_szFileName = FileName;
	mciOpenParms.lpstrDeviceType = "waveaudio";
	mciOpenParms.lpstrElementName = m_szFileName;
	mciSaveParms.lpfilename = m_szFileName;

	if (m_dwReturn = mciSendCommand(0, MCI_OPEN, dwFlags,
									(DWORD)(LPVOID) &mciOpenParms))
	{
		CHAR sErr[250];
		if(mciGetErrorString(m_dwReturn, sErr, 250))
		{
			TRACE("\n************ MCI error is: %s ************\n", sErr);
		}
		m_iState	=	MPS_CLOSED;
		return m_dwReturn;
    }
	m_iState	=	MPS_OPENED;

    // The device opened successfully; get the device ID.
    m_iDeviceID		=	mciOpenParms.wDeviceID;

	m_iCountTracks	=	GetCountTracks();
	m_iCurrentTrack	=	1;

	SetDeviceInfo(MCI_SET_TIME_FORMAT,MCI_FORMAT_MILLISECONDS);

	return m_dwReturn;
}
